/*
* Copyright (C) 2020, KylinSoft Co., Ltd.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
*
*/

#ifndef TITLEBAR_H
#define TITLEBAR_H

#include "theme.h"
#include "about.h"
#include "scanner.h"

#include <QDialog>
#include <QWidget>
#include <QLabel>
#include <QPushButton>
#include <QHBoxLayout>
#include <QEvent>
#include <QMouseEvent>
#include <QDebug>
#include <QPainter>
#include <QToolButton>
#include <QMenu>

#define MAINWINDOW_WIDTH 860
#define MAINWINDOW_HEIGHT 680

class KYCTitleBarDialog : public QDialog
{
    Q_OBJECT

public:
    explicit KYCTitleBarDialog(QWidget *parent = nullptr);
    ~KYCTitleBarDialog();

    void initWindow();
    void initLayout();
    void initStyle();
    void initConnect();

    void setMainWindowAttribute(int w, int h);

protected:
    // 双击标题栏进行界面的最大化/还原
    virtual void mouseDoubleClickEvent(QMouseEvent *event);

    // 进行鼠界面的拖动
    virtual void mouseMoveEvent(QMouseEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);

    // 设置界面标题与图标
    virtual bool eventFilter(QObject *obj, QEvent *event);
    virtual void keyPressEvent(QKeyEvent *e);


private:
    // 最大化/还原
    void updateMaximize();

    bool mMoving;
    bool flagMaxWindow;
    QPoint mLastMousePosition;
    QPoint offset;
    QStringList stylelist;
    QStringList iconthemelist;

    QGSettings *style_settings;
    QGSettings *icon_theme_settings;
    QLabel *m_logo;
    QLabel *m_logoMsg;
    QPushButton *m_pMinimizeButton;
    QPushButton *m_pMaximizeButton;
    QPushButton *m_pCloseButton;
    QToolButton *m_pMenuButton;
    QMenu       *m_pMenu;

    QHBoxLayout *pTitleLayout;
    QHBoxLayout *pButtonLayout;
    QHBoxLayout *pLayout;

    int mainWindowWidth;
    int mainWindowHeight;

private slots:
    // 进行最小化、最大化/还原、关闭操作
    void onClickedMinButton();
    void onClickedMaxButton();
    void onClickedCloseButton();
    void titlebar_icon_theme_changed(QString); // 系统图标主题风格变化
    void titlebar_style_changed(QString); // 系统黑白主题样式变换

Q_SIGNALS:
    void isNormal();
    void isMax();
    void showAboutDialog();
    void isExit();
};
#endif // TITLEBAR_H
